@@ -105,6 +105,7 @@ class RedirectedText(tk.Text):
         self.redir = WidgetRedirector(self)
         self.orig_insert = self.redir.register("insert", self.new_insert)
         self.orig_delete = self.redir.register("delete", self.new_delete)
+        self.fonts = [kw['font']]
 
     def new_insert(self, *args):
         self.orig_insert(*args)
@@ -114,6 +115,23 @@ class RedirectedText(tk.Text):
         self.orig_delete(*args)
         self.event_generate('<<Change>>')
 
+
+class HelpBindings(tk.Toplevel):
+    def __init__(self, parent=None):
+        tk.Toplevel.__init__(self, parent)
+        self.title("Help | Bindings")
+
+        import bindings
+
+        msg = tk.Text(self, width=80, wrap=tk.NONE)
+        msg.insert(tk.END, bindings.description)
+        msg.config(state=tk.DISABLED)
+        msg.pack()
+
+        button = tk.Button(self, text="Dismiss", command=self.destroy)
+        button.pack()
+
+
 #########################################################################
 class StatusBar(tk.Frame):
     """Adapted from the tkinterbook.
@@ -209,6 +227,8 @@ class NotesList(tk.Frame):
         # list containing tuples with each note's title, tags,
         self.note_headers = []
 
+        self.fonts = [f, italic_font, bold_font]
+
     def append(self, note, config):
         """
         @param note: The complete note dictionary.
@@ -415,8 +435,7 @@ class TriggeredcompleteEntry(tk.Entry):
     ctrl-space. The first matching tag is shown. The user can either continue
     pressing ctrl-space to see more matching tags, or right arrow to select
     the current suggestion and continue typing. Backspace will delete the
-    suggested part. Left arrow will delete the last character with the cursor
-    at the end of the line, or the suggested part if not.
+    suggested part. 
 
     To enable triggeredcompletion use set_completion_list(list) to define 
     a list of possible strings to hit.
@@ -434,6 +453,7 @@ class TriggeredcompleteEntry(tk.Entry):
         self._completion_list = completion_list
         self._hits = []
         self._hit_index = 0
+        self.wstart = 0
         self.position = 0
         self.cycle = 0
 
@@ -448,14 +468,23 @@ class TriggeredcompleteEntry(tk.Entry):
 
         else: # set position to end so selection starts where textentry ended
             self.position = len(self.get())
+            wstartsc = self.get().rfind(':')
+            wstartsp = self.get().rfind(' ')
+            if wstartsc < 0 and wstartsp < 0:
+                self.wstart = 0
+            elif wstartsc > wstartsp:
+                self.wstart = wstartsc + 1
+            else:
+                self.wstart = wstartsp + 1
+
             # collect hits
             _hits = []
             for element in self._completion_list:
                 if self.case_sensitive == 0: 
-                    if element.lower().startswith(self.get().lower()):
+                    if element.lower().startswith(self.get()[self.wstart:].lower()):
                          _hits.append(element)
                 else:
-                    if element.startswith(self.get()):
+                    if element.startswith(self.get()[self.wstart:]):
                          _hits.append(element)
 
             self._hit_index = 0
@@ -463,8 +492,8 @@ class TriggeredcompleteEntry(tk.Entry):
 
         # now finally perform the triggered completion
         if self._hits:
-            self.delete(0,tk.END)
-            self.insert(0,self._hits[self._hit_index])
+            self.delete(self.wstart,tk.END)
+            self.insert(self.wstart,self._hits[self._hit_index])
             self.select_range(self.position,tk.END)
 
     def handle_keyrelease(self, event):
@@ -478,19 +507,13 @@ class TriggeredcompleteEntry(tk.Entry):
                 self.delete(self.index(tk.INSERT), tk.END)
                 self.position = self.index(tk.END)
 
-            if event.keysym == "Left":
-                self.cycle = 0
-                if self.position < self.index(tk.END): # delete the selection
-                    self.delete(self.position, tk.END)
-
-                else:
-                    self.position = self.position-1 # delete one character
-                    self.delete(self.position, tk.END)
-
             if event.keysym == "Right":
                 self.position = self.index(tk.END) # go to end (no selection)
                 self.cycle = 0
 
+            if event.keysym == "Left":
+                self.cycle = 0
+
         if event.keysym == "space" and ctrl:
             # cycle 
             self.triggeredcomplete()
@@ -632,7 +655,10 @@ class View(utils.SubjectMixin):
         self.root.protocol('WM_DELETE_WINDOW', self.handler_close)
 
         self.root.bind_all("<Control-g>", lambda e: self.tags_entry.focus())
-        
+        self.root.bind_all("<Control-question>", lambda e: self.cmd_help_bindings())
+        self.root.bind_all("<Control-plus>", lambda e: self.cmd_font_size(+1))
+        self.root.bind_all("<Control-minus>", lambda e: self.cmd_font_size(-1))
+
         self.notes_list.bind("<<NotesListSelect>>", self.cmd_notes_list_select)
         # same behaviour as when the user presses enter on search entry:
         # if something is selected, focus the text area
@@ -778,6 +804,10 @@ class View(utils.SubjectMixin):
 
         help_menu.add_command(label = "About", underline = 0,
                               command = self.cmd_help_about)
+        help_menu.add_command(label = "Bindings", underline = 0,
+                              command = self.cmd_help_bindings,
+                              accelerator="Ctrl+?")
+
 
         # END MENU ######################################################
 
@@ -821,11 +851,28 @@ class View(utils.SubjectMixin):
         
         search_entry.make_style()
         self.search_entry_var = tk.StringVar()
-        #self.search_entry = tk.Entry(search_frame, textvariable=self.search_entry_var, style="Search.entry")
         self.search_entry = TriggeredcompleteEntry(search_frame, self.config.case_sensitive, textvariable=self.search_entry_var, style="Search.entry")
-        #self.search_entry.set_completion_list(self.taglist)
         self.search_entry_var.trace('w', self.handler_search_entry)
+
+        cs_label = tk.Label(search_frame,text="CS ")
+        self.cs_checkbutton_var = tk.IntVar()
+        cs_checkbutton = tk.Checkbutton(search_frame, variable=self.cs_checkbutton_var)
+        self.cs_checkbutton_var.trace('w', self.handler_cs_checkbutton)
+
+        self.search_mode_options = ("gstyle", "regexp")
+        self.search_mode_var = tk.StringVar()
+        # I'm working with ttk.OptionVar, which has that extra default param!
+        self.search_mode_cb = tk.OptionMenu(search_frame, self.search_mode_var,
+            self.search_mode_options[0], *self.search_mode_options)
+        self.search_mode_cb.config(width=6)
+        self.search_mode_var.trace('w', self.handler_search_mode)
+
+        self.search_mode_cb.pack(side=tk.RIGHT, padx=5)
+        cs_checkbutton.pack(side=tk.RIGHT)
+        cs_label.pack(side=tk.RIGHT)
         self.search_entry.pack(fill=tk.X,padx=5, pady=5)
+
+
         search_frame.pack(side=tk.TOP, fill=tk.X)
         
         
@@ -866,7 +913,8 @@ class View(utils.SubjectMixin):
             yscrollbar.pack(side=tk.RIGHT, fill=tk.Y)
 
             #f = tkFont.nametofont('TkFixedFont')
-            f = tkFont.Font(family=self.config.font_family, size=-self.config.font_size)
+            f = tkFont.Font(family=self.config.font_family,
+                            size=self.config.font_size)
             # tkFont.families(root) returns list of available font family names
             # this determines the width of the complete interface (yes)
             text = RedirectedText(master, height=25, width=TEXT_WIDTH,
@@ -888,7 +936,7 @@ class View(utils.SubjectMixin):
 
         # setup user_text ###############################################
         self.text_note = create_scrolled_text(right_frame)
-
+        self.fonts = self.notes_list.fonts + self.text_note.fonts
 
         # finish UI creation ###########################################
 
@@ -934,6 +982,10 @@ class View(utils.SubjectMixin):
             'A rather ugly but cross-platform simplenote client.' % (self.config.app_version,),
             parent = self.root)
 
+    def cmd_help_bindings(self):
+        h = HelpBindings()
+        self.root.wait_window(h)
+
     def cmd_rest(self, event=None):
         self.notify_observers('command:rest', None)
 
@@ -942,7 +994,15 @@ class View(utils.SubjectMixin):
         
     def cmd_sync_full(self, event=None):
         self.notify_observers('command:sync_full', None)
-        
+
+    def cmd_font_size(self, inc_size):
+        for f in self.fonts:
+            f.configure(size=f['size'] + inc_size)
+
+    def handler_cs_checkbutton(self, *args):
+        self.notify_observers('change:cs',
+            utils.KeyValueObject(value=self.cs_checkbutton_var.get()))
+
     def handler_housekeeper(self):
         # nvPY will do saving and syncing!
         self.notify_observers('keep:house', None)
@@ -1041,6 +1101,20 @@ class View(utils.SubjectMixin):
         self.notify_observers('change:entry',
                               utils.KeyValueObject(value=self.search_entry_var.get()))
 
+    def handler_search_mode(self, *args):
+        """
+        Called when the user changes the search mode via the OptionMenu.
+
+        This will also be called even if the user reselects the same option.
+
+        @param args:
+        @return:
+        """
+
+        self.notify_observers('change:search_mode',
+            utils.KeyValueObject(value=self.search_mode_var.get()))
+
+
     def handler_tags_entry(self, *args):
         self.notify_observers('change:tags',
             utils.KeyValueObject(value=self.tags_entry_var.get()))
@@ -1062,16 +1136,17 @@ class View(utils.SubjectMixin):
         
         del self.text_tags_search[:]
         
-        st = self.get_search_entry_text()
+        st = self.notes_list_model.match_regexp
         if not st:
             return
         
         # take care of invalid regular expressions...
         try:
-            if self.config.case_sensitive == 0: 
+            if self.config.case_sensitive == 0:
                 pat = re.compile(st, re.I)
             else:
                 pat = re.compile(st)
+
         except re.error:
             return
         
@@ -1175,6 +1250,30 @@ class View(utils.SubjectMixin):
         self.mute('change:text')
         self.mute('change:tags')
         self.mute('change:pinned')
+
+
+    def set_cs(self, cs, silent=False):
+        if silent:
+            self.mute('change:cs')
+
+        self.cs_checkbutton_var.set(cs)
+
+        self.unmute('change:cs')
+
+    def set_search_mode(self, search_mode, silent=False):
+        """
+
+        @param search_mode: the search mode, "gstyle" or "regexp"
+        @param silent: Specify True if you don't want the view to trigger any events.
+        @return:
+        """
+
+        if silent:
+            self.mute('change:search_mode')
+
+        self.search_mode_var.set(search_mode)
+
+        self.unmute('change:search_mode')
         
     def set_status_text(self, txt):
         self.statusbar.set_status(txt)
